<?php
/*

 Website Baker Project <http://www.websitebaker.org/>
 Copyright (C) 2004-2006, Ryan Djurovich

 Website Baker is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 Website Baker is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Website Baker; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

use bin\{WbAdaptor,SecureTokens,Sanitize};
use bin\helpers\{PreCheck,msgQueue};
use src\Security\{CsfrTokens,Randomizer};
use src\Interfaces\Requester;
use bin\Requester\HttpRequester;

if (defined('SYSTEM_RUN')) {

    $sAddonFile   = \str_replace('\\','/',__FILE__);
    $sAddonPath   = (\dirname($sAddonFile)).'/';
    $sModulesPath = \dirname($sAddonPath).'/';
    $sModuleName  = basename($sModulesPath);
    $sAddonName   = basename($sAddonPath);
    $ModuleRel    = ''.$sModuleName.'/';
    $sAddonRel    = ''.$sModuleName.'/'.$sAddonName.'/';
    $sPattern     = "/^(.*?\/)".$sModuleName."\/.*$/";
    $sAppPath     = preg_replace ($sPattern, "$1", $sModulesPath, 1 );

    try {

        $msg = [];
        $iErr = false;
        $sResult = '';
        $sErrorMsg = null;

        $oReg = WbAdaptor::getInstance();
        $oDb      = $oReg->getDatabase();
        $oTrans   = $oReg->getTranslate();
        $oRequest = $oReg->getRequester();
        $oApp     = $oReg->getApplication();
        $isAuth   = $oApp->is_authenticated();

        $sCallingScript = $oRequest->getServerVar('SCRIPT_NAME');
        $globalStarted = preg_match('/upgrade\-script\.php$/', $sCallingScript);

        $sWbVersion = ($globalStarted && \defined('VERSION') ? VERSION : $oReg->AppVersion);
        $sModulePlatform = PreCheck::getAddonVariable($sAddonName,'platform');
        if (version_compare($sWbVersion, $sModulePlatform, '<')){
            $msg[] = $sErrorMsg = sprintf('It is not possible to install from WebsiteBaker Versions before %s',$sModulePlatform);
            if ($globalStarted){
                echo $sErrorMsg;
            }else{
                throw new Exception ($sErrorMsg);
            }
        } else {

            $MediaPath      = $oReg->AppPath.$oReg->MediaDir;
            $ModuleUrl      = $oReg->AppUrl.$ModuleRel;
            $sAddonUrl      = $oReg->AppUrl.$sAddonRel;
            $sDomain        = $oApp->getDirNamespace(__DIR__);
            $oTrans->enableAddon($sDomain);
            $aLang          = $oTrans->getLangArray();//not needed with twig

            if (is_readable(WB_PATH.'/modules/droplets/droplets.functions.php')) {
                require WB_PATH.'/modules/droplets/droplets.functions.php';
                $sDropletFiles = $sAddonPath.'Droplets/eventlist.php';
                $aDropletFiles = [$sDropletFiles];
                if (is_readable($sDropletFiles)) {
                    $bOverwriteDroplets = true;
                    insertDropletFile($aDropletFiles, $database, $admin,$msg,$bOverwriteDroplets);
                }
            }

            $sInstallStructFile = $sAddonPath.'install-struct.sql.php';
            if (!is_readable($sInstallStructFile)) {
                $sErrorMsg .= \sprintf('<b>\'missing or not readable file ['.basename($sInstallStructFile).']\'</b> ').PHP_EOL;
                $iErr = true;
            }

            //$sSqlMode = $database->get_one('SELECT @@sql_mode;');
            // create tables from sql struct file
            $sInstallStructFile = $sAddonPath.'install-struct.sql.php';
            if (($iErr==false) && !$oDb->SqlImport($sInstallStructFile, $oReg->TablePrefix, 'install' )) {
                $sErrorMsg .= \sprintf('<b>\Error [%d]</b> %s ',$oDb->get_errno(),$oDb->get_error()).PHP_EOL;
                $iErr = true;
            }

        // Create needed addon directories -------------------------------------------
            if (!\is_dir($MediaPath.'calendar/')){ make_dir($MediaPath.'calendar/');}
            if (!\is_dir($sAddonPath.'sql') && make_dir($sAddonPath.'sql')){;}
            if (!\is_dir($sAddonPath.'.images') && make_dir($sAddonPath.'/.images')){;}

            if (!empty($sErrorMsg)) {
                throw new \Exception (\sprintf('%s', $sErrorMsg));
            }
        }
    } catch (\Exception $ex) {
        $sErrMsg = Precheck::xnl2br(\sprintf('[%d] %s', $ex->getLine(), $ex->getMessage()));
        echo $sErrMsg.'<br />';
    }

}

