<?php
/*

 Website Baker Project <http://www.websitebaker.org/>
 Copyright (C) 2004-2006, Ryan Djurovich

 Website Baker is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 Website Baker is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Website Baker; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

use bin\{WbAdaptor,SecureTokens,Sanitize};
use bin\helpers\{PreCheck,msgQueue};
use src\Security\{CsfrTokens,Randomizer};
use src\Interfaces\Requester;
use bin\Requester\HttpRequester;

if (defined('SYSTEM_RUN')) {

    $sAddonFile   = \str_replace('\\','/',__FILE__);
    $sAddonPath   = (\dirname($sAddonFile)).'/';
    $sModulesPath = \dirname($sAddonPath).'/';
    $sModuleName  = basename($sModulesPath);
    $sAddonName   = basename($sAddonPath);
    $ModuleRel    = ''.$sModuleName.'/';
    $sAddonRel    = ''.$sModuleName.'/'.$sAddonName.'/';
    $sPattern     = "/^(.*?\/)".$sModuleName."\/.*$/";
    $sAppPath     = preg_replace ($sPattern, "$1", $sModulesPath, 1 );

    try {

        $msg = [];
        $iErr = false;
        $sResult = '';
        $sErrorMsg = null;

        $oReg = WbAdaptor::getInstance();
        $oDb      = $oReg->getDatabase();
        $oTrans   = $oReg->getTranslate();
        $oRequest = $oReg->getRequester();
        $oApp     = $oReg->getApplication();
        $isAuth   = $oApp->is_authenticated();

        // calling from core upgrade-script.php
        $sCallingScript = $oRequest->getServerVar('SCRIPT_NAME');
        $globalStarted = preg_match('/upgrade\-script\.php$/', $sCallingScript);
        //

        $sWbVersion = ($globalStarted && \defined('VERSION') ? VERSION : $oReg->AppVersion);
        $sModulePlatform = PreCheck::getAddonVariable($sAddonName,'platform');
        if (version_compare($sWbVersion, $sModulePlatform, '<')){
            $msg[] = $sErrorMsg = sprintf('It is not possible to upgrade from WebsiteBaker Versions before %s',$sModulePlatform);
            if ($globalStarted){
                echo $sErrorMsg;
            }else{
                throw new Exception ($sErrorMsg);
            }
        } else {

            $MediaPath      = $oReg->AppPath.$oReg->MediaDir;
            $ModuleUrl      = $oReg->AppUrl.$ModuleRel;
            $sAddonUrl      = $oReg->AppUrl.$sAddonRel;
            $sDomain        = $oApp->getDirNamespace(__DIR__);
            $oTrans->enableAddon($sDomain);
            $aLang          = $oTrans->getLangArray();//not needed with twig

            if (is_readable(WB_PATH.'/modules/droplets/droplets.functions.php')) {
                require WB_PATH.'/modules/droplets/droplets.functions.php';
                $sDropletFiles = $sAddonPath.'Droplets/eventlist.php';
                $aDropletFiles = [$sDropletFiles];
                if (is_readable($sDropletFiles)) {
                    $bOverwriteDroplets = true;
                    insertDropletFile($aDropletFiles, $oDb, $admin,$msg,$bOverwriteDroplets);
                }
            }

            if (!function_exists('Export_Database')) {require ($sAddonPath.'lib/createBackup.php');}
            $sInstallStructFile = $sAddonPath.'install-struct.sql.php';
//echo \nl2br(\sprintf("---- [%04d] %s \n",__LINE__,$sInstallStructFile));
            if (!is_readable($sInstallStructFile)) {
                $sErrorMsg .= \sprintf('<b>\'missing or not readable file ['.basename($sInstallStructFile).']\'</b> ').PHP_EOL;
                $iErr = true;
            }
/* deprecated --------------------------------------------------
            //$sSqlMode = $oDb->get_one('SELECT @@sql_mode;');
            // create tables from sql dump file
//            $oReg->Db->addReplacement('XTABLE_ENGINE','ENGINE=MyISAM CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci');
//            $oReg->Db->addReplacement('XFIELD_COLLATION','COLLATE utf8mb4_unicode_ci');
 */
            if (\is_readable($sInstallStructFile) && !$oDb->SqlImport($sInstallStructFile, $oReg->TablePrefix, 'upgrade' )) {
                $sErrorMsg .= \sprintf('<b>\Error [%d]</b> %s ',$oDb->get_errno(),$oDb->get_error()).PHP_EOL;
                $iErr = true;
            }

        // Create needed addon directories -------------------------------------------
            if (!\is_dir($MediaPath.'calendar/')){ make_dir($MediaPath.'calendar/');}
            if (!\is_dir($sAddonPath.'sql') && make_dir($sAddonPath.'sql')){;}
            if (!\is_dir($sAddonPath.'.images') && make_dir($sAddonPath.'.images')){;}

        // try to create table if not exists
            $oDb->SqlImport($sInstallStructFile, $oReg->TablePrefix, 'upgrade' );

            $sBackupFilename = '.'.$oReg->DbName.\date("_Y-m-d").date('_d'); // _His
            if ($Msg  = $createBackup(
                          \backup\text\DATA,
                          [
                            $oReg->TablePrefix.'mod_procalendar_actions',
                            $oReg->TablePrefix.'mod_procalendar_eventgroups',
                            $oReg->TablePrefix.'mod_procalendar_settings',
                            ],
                          $oReg->TablePrefix,
                          $sBackupFilename)
                ) {
                    $sResult .= \sprintf('Dump %s %s successfully created', $Msg, $sBackupFilename).PHP_EOL;
            }

            $aFilesToDelete = [
                '/backend.js',
            ];
            PreCheck::deleteFiles($sAddonPath,$aFilesToDelete);
            if (!empty($sErrorMsg)) {
                throw new \Exception (\sprintf('%s', $sErrorMsg));
            }
// show by upgrade-script.php, if listed in addons whitelist
            if ($globalStarted) {
                $sResult .= \sprintf('Upgrade %s successfully finished', $sBackupFilename).PHP_EOL;
                echo preg_replace('/[\n\r]/u', '',nl2br($sResult, !defined('XHTML'))).PHP_EOL;
            }

    $aRemoveList = [
                '/old_files/',
                '/images/',
                '/functions.php.new',
                '/install-struct.sql',
                '/install-struct.php',
                '/install-upgrade.sql',
                '/install-struct.sql.php.bak',
              ];
            PreCheck::deleteFiles($sAddonPath,$aRemoveList);

        }
    } catch (\Exception $ex) {
        $sErrMsg = Precheck::xnl2br(\sprintf('[%d] %s', $ex->getLine(), $ex->getMessage()));
        echo $sErrMsg.'<br />';
    }
}

